"""
Causes an exception to be raised, caught, and logged. This is because there is a None value present in the dictionary
being logged. The other logs in the batch are lost.
"""


def main():
    import logging

    from opentelemetry._logs import set_logger_provider
    from opentelemetry.exporter.otlp.proto.grpc._log_exporter import (
        OTLPLogExporter,
    )
    from opentelemetry.sdk._logs import LoggerProvider, LoggingHandler
    from opentelemetry.sdk._logs.export import BatchLogRecordProcessor

    logger_provider = LoggerProvider()
    set_logger_provider(logger_provider)
    exporter = OTLPLogExporter(insecure=True)
    logger_provider.add_log_record_processor(BatchLogRecordProcessor(exporter))
    handler = LoggingHandler(level=logging.NOTSET, logger_provider=logger_provider)
    logging.getLogger().addHandler(handler)

    # works, but is never exported
    logging.warning({"hello": "world"})
    # causes batch to be discarded
    logging.warning({"hello": None})
    # works, but is never exported
    logging.warning({"goodbye": "world"})

    handler.flush()
    # works since previous batch was flushed already
    logging.warning({"hello": "again"})
    logger_provider.shutdown()


if __name__ == "__main__":
    main()


class MyOtelTest:
    def environment_variables(self):
        return {}

    def requirements(self):
        return ("opentelemetry-distro[otlp]",)

    def wrapper_command(self):
        return ""

    def on_start(self):
        pass

    def on_stop(self, tel, stdout: str, stderr: str, returncode: int):
        return 12

    def is_http(self):
        return False



import argparse

from oteltest.sink import run_grpc, run_http


def main():
    parser = argparse.ArgumentParser(description="OpenTelemetry Python Tester")
    parser.add_argument("--http", action="store_true", help="Use HTTP instead of gRPC")
    args = parser.parse_args()
    if args.http:
        run_http()
    else:
        run_grpc()


if __name__ == "__main__":
    main()



from opentelemetry.proto.collector.logs.v1 import (  # type: ignore
    logs_service_pb2,
    logs_service_pb2_grpc,
)
from opentelemetry.proto.collector.metrics.v1 import (  # type: ignore
    metrics_service_pb2,
    metrics_service_pb2_grpc,
)
from opentelemetry.proto.collector.trace.v1 import (  # type: ignore
    trace_service_pb2,
    trace_service_pb2_grpc,
)


class _LogsServiceServicer(logs_service_pb2_grpc.LogsServiceServicer):
    def __init__(self, handle_request):
        self.handle_request = handle_request

    def Export(self, request, context):  # noqa: N802
        self.handle_request(request, headers_from_context(context))
        return logs_service_pb2.ExportLogsServiceResponse()


class _TraceServiceServicer(trace_service_pb2_grpc.TraceServiceServicer):
    def __init__(self, handle_request):
        self.handle_request = handle_request

    def Export(self, request, context):  # noqa: N802
        self.handle_request(request, headers_from_context(context))
        return trace_service_pb2.ExportTraceServiceResponse()


class _MetricsServiceServicer(metrics_service_pb2_grpc.MetricsServiceServicer):
    def __init__(self, handle_request):
        self.handle_request = handle_request

    def Export(self, request, context):  # noqa: N802
        self.handle_request(request, headers_from_context(context))
        return metrics_service_pb2.ExportMetricsServiceResponse()


def headers_from_context(ctx):
    return {k: v for k, v in ctx.invocation_metadata()}



import threading
from concurrent import futures
from http.server import BaseHTTPRequestHandler, HTTPServer

import grpc  # type: ignore
from opentelemetry.proto.collector.logs.v1 import (  # type: ignore
    logs_service_pb2_grpc,
)
from opentelemetry.proto.collector.logs.v1.logs_service_pb2 import (
    ExportLogsServiceRequest,  # type: ignore
)
from opentelemetry.proto.collector.metrics.v1 import (  # type: ignore
    metrics_service_pb2_grpc,
)
from opentelemetry.proto.collector.metrics.v1.metrics_service_pb2 import (
    ExportMetricsServiceRequest,  # type: ignore
)
from opentelemetry.proto.collector.trace.v1 import (  # type: ignore
    trace_service_pb2_grpc,
)
from opentelemetry.proto.collector.trace.v1.trace_service_pb2 import (
    ExportTraceServiceRequest,  # type: ignore
)

from oteltest.sink.handler import PrintHandler, RequestHandler
from oteltest.sink.private import (
    _LogsServiceServicer,
    _MetricsServiceServicer,
    _TraceServiceServicer,
)


class GrpcSink:
    """
    This is an OTel GRPC server to which you can send metrics, traces, and
    logs. It requires a RequestHandler implementation passed in.
    """

    def __init__(
        self,
        request_handler: RequestHandler,
        max_workers: int = 10,
        address: str = "0.0.0.0:4317",
    ):
        self.svr = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
        trace_service_pb2_grpc.add_TraceServiceServicer_to_server(
            _TraceServiceServicer(request_handler.handle_trace), self.svr
        )
        metrics_service_pb2_grpc.add_MetricsServiceServicer_to_server(
            _MetricsServiceServicer(request_handler.handle_metrics), self.svr
        )
        logs_service_pb2_grpc.add_LogsServiceServicer_to_server(
            _LogsServiceServicer(request_handler.handle_logs), self.svr
        )
        self.svr.add_insecure_port(address)
        print(f"- Set up grpc sink at address {address}")

    def start(self):
        """Starts the server. Does not block."""
        self.svr.start()

    def wait_for_termination(self):
        """Blocks until the server stops."""
        try:
            self.svr.wait_for_termination()
        except BaseException:
            print("terminated")

    def stop(self):
        """Stops the server immediately."""
        self.svr.stop(grace=None)


class HttpSink:

    def __init__(self, listener, port=4318, daemon=True):
        self.listener = listener
        self.port = port
        self.handlers = {
            "/v1/traces": self.handle_trace,
            "/v1/metrics": self.handle_metrics,
            "/v1/logs": self.handle_logs,
        }
        self.svr_thread = threading.Thread(target=self.run_server)
        self.svr_thread.daemon = daemon
        print(f"- Set up http sink on port {port}")

    def start(self):
        self.svr_thread.start()

    def run_server(self):
        class Handler(BaseHTTPRequestHandler):

            # noinspection PyPep8Naming
            def do_POST(this):
                # /v1/traces
                content_length = int(this.headers["Content-Length"])
                post_data = this.rfile.read(content_length)

                otlp_handler_func = self.handlers.get(this.path)
                if otlp_handler_func:
                    # noinspection PyArgumentList
                    otlp_handler_func(
                        post_data, {k: v for k, v in this.headers.items()}
                    )

                this.send_response(200)
                this.send_header("Content-type", "text/html")
                this.end_headers()

                this.wfile.write("OK".encode("utf-8"))

        # noinspection PyTypeChecker
        httpd = HTTPServer(("", self.port), Handler)
        httpd.serve_forever()

    def handle_trace(self, post_data, headers):
        req = ExportTraceServiceRequest()
        req.ParseFromString(post_data)
        self.listener.handle_trace(req, headers)

    def handle_metrics(self, post_data, headers):
        req = ExportMetricsServiceRequest()
        req.ParseFromString(post_data)
        self.listener.handle_metrics(req, headers)

    def handle_logs(self, post_data, headers):
        req = ExportLogsServiceRequest()
        req.ParseFromString(post_data)
        self.listener.handle_logs(req, headers)

    def stop(self):
        self.svr_thread.join()


def run_grpc():
    sink = GrpcSink(PrintHandler())
    sink.start()
    sink.wait_for_termination()


def run_http():
    sink = HttpSink(PrintHandler(), daemon=False)
    sink.start()



from typing import Optional, Sequence, Mapping


class MyOtelTest:
    def environment_variables(self) -> Mapping[str, str]:
        return {}

    def requirements(self) -> Sequence[str]:
        return ("opentelemetry-distro",)

    def wrapper_command(self) -> str:
        return ""

    def on_start(self) -> Optional[float]:
        return None

    def on_stop(self, tel, stdout, stderr, returncode) -> None:
        pass



import argparse

from oteltest.private import run


def main():
    parser = argparse.ArgumentParser(description="OpenTelemetry Python Tester")

    d_help = "An optional override directory to hold per-script venv directories."
    parser.add_argument(
        "-d", "--venv-parent-dir", type=str, required=False, help=d_help
    )

    j_help = "An optional value to hold the directory into which json telemetry files are written"
    parser.add_argument("-j", "--json-dir", type=str, required=False, help=j_help, default="json")

    parser.add_argument(
        "script_paths",
        nargs="+",
        help="One or more oteltest files or directories containing oteltest scripts",
    )

    args = parser.parse_args()
    run(args.script_paths, args.venv_parent_dir, args.json_dir)


if __name__ == "__main__":
    main()



from lib import trace_loop

SERVICE_NAME = "my-otel-test"
NUM_ADDS = 12

if __name__ == "__main__":
    trace_loop(NUM_ADDS)

class MyOtelTest:
    def requirements(self):
        return ("splunk-opentelemetry[otlp]",)

    def environment_variables(self):
        return {
            "OTEL_SERVICE_NAME": SERVICE_NAME
        }

    def wrapper_command(self):
        return "splunk-py-trace"

    def on_start(self):
        return 12

    def on_stop(self, telemetry, stdout: str, stderr: str, returncode: int) -> None:
        print(f"script completed with return code {returncode}")

    def is_http(self):
        return False
